@@ -1,5 +1,5 @@
 
-from . import grace, config, working_directory, io
+from . import grace, config, working_directory, io, spanner
 
 import os, itertools, collections
     
@@ -23,20 +23,37 @@ class Norm_from_samples(config.Action_with_prefix):
         totals = [ 0 ] * len(sample_names)
         sites = 0
         good_sites = 0
+        #for name in chromosome_names:
+        #    grace.status('Normalization '+name)
+        #    iter_unambiguous0 = itertools.izip(*[ item[name].depths[0] for item in depths ])   
+        #    iter_unambiguous1 = itertools.izip(*[ item[name].depths[1] for item in depths ])   
+        #    iter_ambiguous0 = itertools.izip(*[ item[name].ambiguous_depths[0] for item in depths ])   
+        #    iter_ambiguous1 = itertools.izip(*[ item[name].ambiguous_depths[1] for item in depths ])
+        #    for unambiguous0, unambiguous1, ambiguous0, ambiguous1 in itertools.izip(
+        #        iter_unambiguous0, iter_unambiguous1, iter_ambiguous0, iter_ambiguous1):
+        #        sites += 1
+        #        if unambiguous0 != ambiguous0 or unambiguous1 != ambiguous1:
+        #            continue
+        #        good_sites += 1
+        #        for i in xrange(len(sample_names)):
+        #            totals[i] += ambiguous0[i]+ambiguous1[i]
+
         for name in chromosome_names:
             grace.status('Normalization '+name)
-            iter_unambiguous0 = itertools.izip(*[ item[name].depths[0] for item in depths ])   
-            iter_unambiguous1 = itertools.izip(*[ item[name].depths[1] for item in depths ])   
-            iter_ambiguous0 = itertools.izip(*[ item[name].ambiguous_depths[0] for item in depths ])   
-            iter_ambiguous1 = itertools.izip(*[ item[name].ambiguous_depths[1] for item in depths ])
-            for unambiguous0, unambiguous1, ambiguous0, ambiguous1 in itertools.izip(
+            iter_unambiguous0 = spanner.zip_spanners(*[ item[name].depths[0].spanner() for item in depths ])   
+            iter_unambiguous1 = spanner.zip_spanners(*[ item[name].depths[1].spanner() for item in depths ])   
+            iter_ambiguous0 = spanner.zip_spanners(*[ item[name].ambiguous_depths[0].spanner() for item in depths ])   
+            iter_ambiguous1 = spanner.zip_spanners(*[ item[name].ambiguous_depths[1].spanner() for item in depths ])
+            for weight, (unambiguous0, unambiguous1, ambiguous0, ambiguous1) in spanner.zip_spanners(
                 iter_unambiguous0, iter_unambiguous1, iter_ambiguous0, iter_ambiguous1):
-                sites += 1
+                sites += weight
                 if unambiguous0 != ambiguous0 or unambiguous1 != ambiguous1:
                     continue
-                good_sites += 1
+                good_sites += weight
                 for i in xrange(len(sample_names)):
-                    totals[i] += ambiguous0[i]+ambiguous1[i]
+                    totals[i] += weight*(ambiguous0[i]+ambiguous1[i])
+
+
         grace.status('')
         
         self.log.log('Locations used: %s of %s\n' % (grace.pretty_number(good_sites),grace.pretty_number(sites)))
